/*>>> jetbnc.c: Jester benchmark and evaluation */

/*
 *	JETBNC.C
 *  Jester benchmark
 *  Activate from command line with "bnch"
 *
 *  02-15-2001 13:43:43  Create file, convert to longs
 *  02-16-2001 14:50:00  Reformat code, stdtype.h
 *  05-11-2001 10:45:42  Use SAN Kit I/O fns
 *  05-14-2001 11:04:25  Rearrange prototypes
 *  05-15-2001 14:09:32  Disable output during benchmark
 *  05-22-2001  9:40:48  Add command to evaluate position 
 *  05-23-2001 12:18:07  Add play move, re-evaluate, takeback
 *  05-25-2001  9:17:59  Fix how evaluation difference is computed
 */

#include <stdio.h>

#include "stdtype.h"

#ifndef _LINUX
#include "SanKit\sandef.h"
#include "SanKit\sanvar.h"
#include "SanKit\sandsp.h"
#include "SanKit\sanmpu.h"
#include "SanKit\sansel.h"
#include "SanKit\sanmnd.h"
#else
#include "SanKit/sandef.h"
#include "SanKit/sanvar.h"
#include "SanKit/sandsp.h"
#include "SanKit/sanmpu.h"
#include "SanKit/sansel.h"
#include "SanKit/sanmnd.h"
#endif


#include "jetdef.h"
#include "jetdat.h"
#include "jetpro.h"

/* bench: This is a little benchmark code that calculates how many
   nodes per second TSCP searches.
   
   It sets the position to move 17 of Bobby Fischer vs. J. Sherwin,
   New Jersey State Open Championship, 9/2/1957.
   
   Then it searches five ply three times. It calculates nodes per
   second from the best time. */

siT bench_color[64] = 
  {
  6, 1, 1, 6, 6, 1, 1, 6,
  1, 6, 6, 6, 6, 1, 1, 1,
  6, 1, 6, 1, 1, 6, 1, 6,
  6, 6, 6, 1, 6, 6, 0, 6,
  6, 6, 1, 0, 6, 6, 6, 6,
  6, 6, 0, 6, 6, 6, 0, 6,
  0, 0, 0, 6, 6, 0, 0, 0,
  0, 6, 0, 6, 0, 6, 0, 6
  }
;

siT bench_piece[64] = 
  {
  6, 3, 2, 6, 6, 3, 5, 6,
  0, 6, 6, 6, 6, 0, 0, 0,
  6, 0, 6, 4, 0, 6, 1, 6,
  6, 6, 6, 1, 6, 6, 1, 6,
  6, 6, 0, 0, 6, 6, 6, 6,
  6, 6, 0, 6, 6, 6, 0, 6,
  0, 0, 4, 6, 6, 0, 2, 0,
  3, 6, 2, 6, 3, 6, 5, 6
  }
;

void bench()
  {
  siT i;
  liT t[3];
  double nps;
  double mips;
  extern boolT ftime_ok; /* in jester.c */
  void print_board(void); /* in jetxbd.c */

  for (i = 0; i < 64; ++i) 
    {
    color[i] = bench_color[i];
    piece[i] = bench_piece[i];
    }
  side = LIGHT;
  xside = DARK;
  castle = 0;
  ep = -1;
  fifty = 0;
  ply = 0;
  hply = 0;
  print_board();
  max_time = 60000L;
  max_depth = 5;
  for (i = 0; i < 3; ++i) 
    {
    think(0); /* turn off output, so it doesn't call SAN kit */
    t[i] = get_ms() - start_time;
    sprintf(stv, "Time: %ld ms\n", t[i]);
		SANDspSTV();
    }
  if (t[1] < t[0])
    t[0] = t[1];
  if (t[2] < t[0])
    t[0] = t[2];
  SANDspNL();
  sprintf(stv, "Nodes: %ld\n", nodes);
	SANDspSTV();
  sprintf(stv, "Best time: %ld ms\n", t[0]);
	SANDspSTV();
  if (!ftime_ok) 
    {
    SANDspNL();
    SANDspLine("Your compiler's ftime() function is apparently only accurate");
    SANDspLine("to the second. Please change the get_ms() function in main.c");
    SANDspLine("to make it more accurate.");
    SANDspNL();
    return;
    }
  if (t[0] == 0) 
    {
    SANDspLine("(invalid)");
    return;
    }
  nps = (double)nodes / (double)t[0];
  nps *= 1000.0;

  /* I measured the number of instructions it takes to execute the bench()
  function on an SGI workstation with an R10000. I divided that number by
  3 to get the approximate number of instructions that are timed. Divide
  by time (and a million) to get MIPS... So it's not a perfect
  measurement, but it's way more interesting than Dhrystone MIPS. :) */
  mips = 3744411.0827 / (double)t[0];

  sprintf(stv, "Nodes per second: %ld (%.1f MIPS)\n", (long)nps, mips);
	SANDspSTV();
  }

/*
 * Evaluate move in current position.
 * Evaluates position, makes move, re-evaluates position, and
 *   compares the two evaluations.  
 * NOTE Jet engine must be selected, or you'll get 0 back
 */	
void pos_eval(void)
	{
	siT i, flag;
  mptrT mptr;
	float eval1, eval2;

	/* evaluation current position */	
  SANSelect();
	eval1 = ((float)JETLastEval()) / 100.0;

	/* apply candidate move(s) and reevaluate */
	flag = 1;
	i = 1;
	while (flag && (i < tkn)) 
		{
		mptr = SANDecodeAux(tkv[i]);
		if (mptr == NULL)
			{
      SANBadParmForm(i);
			flag = 0;
			}
		else
			{
			SANDspSAN(mptr);
			SANPlayMove(mptr);
			SANSelect();
			eval2 = ((float)JETLastEval()) / 100.0;
			sprintf(stv, "\t%7.2f\n", -(eval2+eval1));
			SANDspSTV();
			SANUnplayMove();
			}
		i++;
	  }


	return;
	}
	
/* jetbnc.c: EOF */
