/*
 *	JETOBK.C
 *  Opening book module
 *
 *  02-20-2001 13:38:49  Creation
 *  05-14-2001 11:06:34  include sandef.h for jetpro.h
 *  02-09-2002 13:16:00  reformat block in FixBookMove()
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stdtype.h"
#ifndef _LINUX
#include "SanKit\sandef.h"
#else
#include "SanKit/sandef.h"
#endif

#include "jetdef.h"
#include "jetdat.h"
#include "jetpro.h"

#define BOOK_LINES 80
#define BOOK_DEPTH 24
#define MAX_BOOK_LINE 255 			/* longest title in opening book */
#define OPENING_TAG "opening"

static char bookbuf[MAX_BOOK_LINE]; /* buf read from book */

static move opening_book[BOOK_LINES][BOOK_DEPTH];
static siT book_lines;

/*
 * Parse book move in buffer s into move m
 * Move in s is in coordinate notation, i.e. e2e4
 */
move ParseBookMove(char s[])
  {
  move m; 

  m.b.from = s[0] - 'a';
  m.b.from += 8 * (8 - (s[1] - '0'));
  m.b.to = s[2] - 'a';
  m.b.to += 8 * (8 - (s[3] - '0'));
  return m;
  }

/*
 * Load opening book.  Passed filename as string.
 * Returns boolean set true if successful.  It's OK if book is not loaded.
 * See file DEFAULT_OPENING_BOOK for format, it's basically a list of
 * 	move pairs.  Note that format is very specific, and two spaces are
 *  required between pairs.
 */
boolT LoadOpeningBook(charptrT filename)
  {
  move m;
  FILEptrT ob;
  siT mv;

  memset(opening_book, 0, sizeof(opening_book));    
  ob = fopen(filename, "r");
  if (ob != NULL)
    {
    book_lines = -1;
    mv = 0;
    while (!feof(ob))
      {
      fgets(bookbuf, MAX_BOOK_LINE, ob);
      if (!strncmp(bookbuf, OPENING_TAG, strlen(OPENING_TAG)))
        {
        mv = 0;
        book_lines++;
        }
      else
        {
        m = ParseBookMove(&bookbuf[0]);
        opening_book[book_lines][mv++] = m;
        m = ParseBookMove(&bookbuf[6]);
        opening_book[book_lines][mv++] = m;
        }
      }
    fclose(ob);
    return TRUE;
    }
  else
    {
    return FALSE;
    }  
  }

/*
 * Turn a book move into a "real" move.
 * Generate moves and then try to match the one sent.
 * Generating the moves is probably unnecessary, but this
 * works.
 */
move FixBookMove(move obm)
  {
  siT i;

  ply = 0;
  gen(); 
  for (i = 0; i < first_move[1]; ++i)
		{
    if (gen_dat[i].m.b.from == obm.b.from && 
      gen_dat[i].m.b.to == obm.b.to)
      return gen_dat[i].m;
		}
	return obm; /* should never be reached */
  }

/*
 * Return true if two moves are equal.
 * Means their 'from' and 'to' squares are equal.
 * May be profitably used elsewhere.
 */
boolT is_equal_move(move x, move y)
  {
  return ((x.b.from == y.b.from) && 
    (x.b.to   == y.b.to))   ? TRUE : FALSE;
  }

/*
 * Lookup move in opening book.
 * Return move in book, or "null" move.
 */
move lookup(void)
  {
  int r, r0;
  boolT found;
  siT i, j, depth;
  move book, played, themove;

  r0 = 0;
  found = FALSE;
  themove.u = 0L;
  depth = min(hply, BOOK_DEPTH);
  for (i = 0; i < book_lines; i++)
    {
    for (j = 0; j < depth; j++)
      {
      played = hist_dat[j].m;
      book = opening_book[i][j];
      if (!book.b.from && !book.b.to)
        break;
      if (!is_equal_move(played, book))
        break;
      }
    if (j >= depth)
      {
      r = rand();
      if (r > r0)
        {
        r0 = r;
        themove = opening_book[i][depth];
        found = TRUE;
        }
      }
    }
  in_book = found;
  return themove;	
  }

/* jetobk.c: EOF */
