/*
 *	JETSRC.C
 *  Jester search algorithms
 *
 *  02-08-2001  9:52:53  correct display for liT nodes
 *  02-09-2001  9:17:37  start_time, stop_time now long
 *  02-12-2001 10:51:42  new source names, stdtype.h
 *             11:31:01  make score long
 *  02-15-2001 15:27:18  search() and quiesce() return longs
 *  02-16-2001 14:53:37  reformat, stdtype.h
 *             17:00:00  fix think() xboard format output
 *  02-20-2001 16:39:55  call lookup() to search opening book
 *             16:47:52  fix printf() format in think()
 *  03-09-2001 15:34:11  check for pending chars in checkup()
 *  03-12-2001 12:31:38  code to set/reset stdin raw mode
 *  03-13-2001 11:40:25  call tty_raw(), not tty_cbreak()
 *  05-11-2001 10:14:04  remove tty... stuff; add SAN Kit I/O fns
 *  05-11-2001 14:11:12  remove fflush(stdout)
 *  05-14-2001 10:06:28  add showSANPV()
 *             13:45:48  toggle namd option to make output nice
 *  05-16-2001  9:32:02  toggle nabd option, too
 *  05-22-2001  9:33:14  save x in think() as evaluation
 *  05-24-2001 12:25:55  add forced move caching
 *  12-19-2001 14:06:00  put back ch_pending() call
 *  01-12-2002 11:16:00  add elapsed time in think() display
 *  02-06-2002 19:00:00  clean up showSANPV()
 *  02-08-2002 18:47:00  use JETtoSAN() in showSANPV()
 *  02-08-2002 20:14:00  move memset(pv...) in think()
 *  02-09-2002 13:15:00  tighten check for end of book move 
 */


#include <stdio.h>
#include <string.h>

#include "stdtype.h"
#ifndef _LINUX
#include "SanKit\sandef.h"
#include "SanKit\sanvar.h"
#include "SanKit\sandsp.h"
#include "SanKit\sanmpu.h"
#else
#include "SanKit/sandef.h"
#include "SanKit/sanvar.h"
#include "SanKit/sandsp.h"
#include "SanKit/sanmpu.h"
#endif


#include "jetdef.h"
#include "jetdat.h"
#include "jetpro.h"

#define CHECKUP_TIME (16384-1)

static liT last_eval;
static siT deepest;

/* see beginning of think() */
#include <setjmp.h>
jmp_buf env;
boolT stop_search;

/*
 * Return last evaluation result to caller 
 * Used for scanning and proofing moves
 * NOTE  be careful that this is called *immediately* after
 *  think()
 */
liT JETLastEval(void)
	{
	return last_eval;
	}
	
/* 
 * Display PV as series of moves
 * Called between plies of the search
 */
void showSANPV(void)
	{
  mT SANmove;

	SANDspStr(side == LIGHT ? " " : " ...");
  JETtoSAN(pv[0][0].b, &SANmove);
  SANDspSAN(&SANmove);
//SANDspStr(move_str(pv[0][0].b));
	SANDspNL();
	}
	
/*
 * force_cache() checks the move selected on exit from think()
 *   to see if it is a capture.  If it is, it and it's response 
 *   are saved so time is saved in a match.  (Xboard only)
 * forced and forced_reply are set, or NULL on exit
 */
void force_cache(void)
	{
	forced.u = 0L;
	forced_reply.u = 0L;
	if (pv_length[deepest] > 2) 
		{
		if (pv[0][1].b.bits & 0x01) /* capture */
			{
			forced = pv[0][1];
			forced_reply = pv[0][2];
	    }
	  }
	}
	
/*	
 * think() calls search() iteratively. Search statistics
 * are printed depending on the value of output:
 *  0 = no output
 *  1 = normal output
 *  2 = xboard format output 
 */	 
void think(siT output)
  {
  siT i;
  move bookmove;
	
	/* all of these have been moved (debug) */
  ply = 0;
  nodes = 0L;
	deepest = 0;
  memset(pv, 0, sizeof(pv)); // moved
  memset(history, 0, sizeof(history));
  
  /* if we're in book, try to lookup a move and return that */
  if (in_book)
    {
		bookmove = FixBookMove(lookup());
		if (in_book && (bookmove.b.from > 0) && (bookmove.b.to > 0))
	  	{
		  pv[0][0] = bookmove;
		  return;
		  }
		}

  /* some code that lets us longjmp back here and return
     from think() when our time is up */
	
  stop_search = FALSE;
  setjmp(env);
  if (stop_search)
    {
    /* make sure to take back line we were searching */
		deepest = ply;
    while (ply)
      takeback();
		force_cache();
    return;
    }

	/* old init here... (debug) */

  start_time = (long)get_ms();
  stop_time = start_time + max_time;

  if (output == 1)
    SANDspStr("ply      nodes  score   secs.  pv\n");
  for (i = 1; i <= max_depth; ++i)
    {
    follow_pv = TRUE;
    last_eval= search(-10000, 10000, i);
    if (output == 1)
			{
      sprintf(stv, "%3d  %9ld  %5ld  %6.1f ", i, nodes, last_eval, 
      	(float)((long)get_ms()-start_time)/1000.0);
			SANDspSTV();
			}
    else if (output == 2)
			{
      sprintf(stv, "%d %ld %ld %ld",
        i, last_eval, (get_ms() - start_time) / 10, nodes);
			SANDspSTV();
			}
    if (output)
			showSANPV();
    if (last_eval > 9000 || last_eval < -9000)
      break;
    }
	deepest = max_depth;
	force_cache();
  }


/* search() does just that, in negamax fashion */

liT search(liT alpha, liT beta, siT depth)
  {
  liT x;
  siT i, j;
  boolT c, f;

  /* we're as deep as we want to be; call quiesce() to get
     a reasonable score and return it. */
  if (!depth)
    return quiesce(alpha,beta);
  ++nodes;

  /* do housekeeping every 1024 nodes */
  if ((nodes & CHECKUP_TIME) == 0)
    checkup();

  pv_length[ply] = ply;

  /* if this isn't the root of the search tree (where we have
  to pick a move and can't simply return 0) then check to
  see if the position is a repeat. if so, we can assume that
  this line is a draw and return 0. */
  if (ply && reps())
    return 0;

  /* are we too deep? */
  if (ply >= MAX_PLY - 1)
    return eval();
  if (hply >= HIST_STACK - 1)
    return eval();

  /* are we in check? if so, we want to search deeper */
  c = in_check(side);
  if (c)
    ++depth;
  gen();
  if (follow_pv)  /* are we following the PV? */
    sort_pv();
  f = FALSE;

  /* loop through the moves */
  for (i = first_move[ply]; i < first_move[ply + 1]; ++i)
    {
    sort(i);
    if (!makemove(gen_dat[i].m.b))
      continue;
    f = TRUE;
    x = -search(-beta, -alpha, depth - 1);
    takeback();
    if (x > alpha)
      {

      /* this move caused a cutoff, so increase the history
      value so it gets ordered high next time we can
      search it */
      history[(int)gen_dat[i].m.b.from][(int)gen_dat[i].m.b.to] += depth;
      if (x >= beta)
        return beta;
      alpha = x;

      /* update the PV */
      pv[ply][ply] = gen_dat[i].m;
      for (j = ply + 1; j < pv_length[ply + 1]; ++j)
        pv[ply][j] = pv[ply + 1][j];
      pv_length[ply] = pv_length[ply + 1];
      }
    }

  /* no legal moves? then we're in checkmate or stalemate */
  if (!f)
    {
    if (c)
      return -10000 + ply;
    else
      return 0;
    }

  /* fifty move draw rule */
  if (fifty >= 100)
    return 0;
  return alpha;
  }


/* quiesce() is a recursive minimax search function with
   alpha-beta cutoffs. In other words, negamax. It basically
   only searches capture sequences and allows the evaluation
   function to cut the search off (and set alpha). The idea
   is to find a position where there isn't a lot going on
   so the static evaluation function will work. */

liT quiesce(liT alpha, liT beta)
  {
  liT x;
  siT i, j;

  ++nodes;

  /* do housekeeping every 1024 nodes */
  if ((nodes & CHECKUP_TIME) == 0)
    checkup();

  pv_length[ply] = ply;

  /* are we too deep? */
  if (ply >= MAX_PLY - 1)
    return eval();
  if (hply >= HIST_STACK - 1)
    return eval();

  /* check with the evaluation function */
  x = eval();
  if (x >= beta)
    return beta;
  if (x > alpha)
    alpha = x;

  gen_caps();
  if (follow_pv)  /* are we following the PV? */
    sort_pv();

  /* loop through the moves */
  for (i = first_move[ply]; i < first_move[ply + 1]; ++i)
    {
    sort(i);
    if (!makemove(gen_dat[i].m.b))
      continue;
    x = -quiesce(-beta, -alpha);
    takeback();
    if (x > alpha)
      {
      if (x >= beta)
        return beta;
      alpha = x;

      /* update the PV */
      pv[ply][ply] = gen_dat[i].m;
      for (j = ply + 1; j < pv_length[ply + 1]; ++j)
        pv[ply][j] = pv[ply + 1][j];
      pv_length[ply] = pv_length[ply + 1];
      }
    }
  return alpha;
  }


/* reps() returns the number of times that the current
   position has been repeated. Thanks to John Stanback
   for this clever algorithm. */

siT reps()
  {
  siT i;
  siT b[64];
  siT c = 0;  /* count of squares that are different from
                 the current position */
  siT r = 0;  /* number of repetitions */

  /* is a repetition impossible? */
  if (fifty <= 3)
    return 0;

  memset(b, 0, sizeof(b));

  /* loop through the reversible moves */
  for (i = hply - 1; i >= hply - fifty - 1; --i)
    {
    if (++b[hist_dat[i].m.b.from] == 0)
      --c;
    else
      ++c;
    if (--b[hist_dat[i].m.b.to] == 0)
      --c;
    else
      ++c;
    if (c == 0)
      ++r;
    }

  return r;
  }


/* sort_pv() is called when the search function is following
   the PV (Principal Variation). It looks through the current
   ply's move list to see if the PV move is there. If so,
   it adds 10,000,000 to the move's score so it's played first
   by the search function. If not, follow_pv remains FALSE and
   search() stops calling sort_pv(). */

void sort_pv()
  {
  siT i;

  follow_pv = FALSE;
  for(i = first_move[ply]; i < first_move[ply + 1]; ++i)
    if (gen_dat[i].m.u == pv[0][ply].u)
      {
      follow_pv = TRUE;
      gen_dat[i].score += 10000000L;
      return;
      }
  }


/* sort() searches the current ply's move list from 'from'
   to the end to find the move with the highest score. Then it
   swaps that move and the 'from' move so the move with the
   highest score gets searched next, and hopefully produces
   a cutoff. */

void sort(siT from)
  {
  siT i;
  liT bs;  /* best score */
  siT bi;  /* best i */
  gen_t g;

  bs = -1L;
  bi = from;
  for (i = from; i < first_move[ply + 1]; ++i)
    if (gen_dat[i].score > bs)
      {
      bs = gen_dat[i].score;
      bi = i;
      }
  g = gen_dat[from];
  gen_dat[from] = gen_dat[bi];
  gen_dat[bi] = g;
  }

/* 
 * checkup() is called once in a while during the search. 
 * Checks to see if time is up, or a character is pending on stdin
 */
void checkup()
  {
  /* is the engine's time up? if so, longjmp back to the
     beginning of think() */
  if (get_ms() >= stop_time || ch_pending())
    {
    stop_search = TRUE;
    longjmp(env, 0);
    }
  }

/* jetsrc.c: EOF */
