/*>>> sanosi.c: SAN project operating system interface */

/* Revised: 05-11-2001 14:31:14 */

/* 
 * 05-10-2001 17:15:16  Creation 
 * 05-11-2001 11:32:04  Add SANfgets()
 * 05-11-2001 14:07:33  Add SANXboardInit() and SANXboardTerm()
 * 05-11-2001 14:10:34  Flush stdout if in xboard mode
 * 02-11-2002 20:36:00  Win32 compatible is_ready() courtesy Dann Corbit
 */


/* The functions in this module handle character I/O to the
operating system. The functions in SANDSP.C vector through these
functions, which in turn may be re-directed as needed to support
Winboard, etc. 

This is for all I/O that goes to the console only. It is *not*
for reading commands, which may come from a redirected file, nor
PGN files, etc. 

You can use the `in_xboard' flag to determine if the program is
currently running through Winboard/Xboard, or the "standard"
console. */

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <time.h>

#include "stdtype.h"

#ifndef _LINUX
#include "SanKit\sandef.h"
#include "SanKit\sanvar.h"
#include "SanKit\sanosi.h"
#else
#include "SanKit/sandef.h"
#include "SanKit/sanvar.h"
#include "SanKit/sanosi.h"
#endif


/*
 * Replacement for putc()
 * flush(stdout) if in xboard mode and character is a newline
 */
void SANputc(char c)
	{
	fputc(c, stdout);
	if (in_xboard && (c == '\n')) 
		fflush(stdout);
	}
	
/*
 * Replacement for puts()
 */
void SANputs(charptrT s)
	{
	fprintf(stdout, "%s", s);
	if (in_xboard) 
		fflush(stdout);
	}
	
/*
 * Replacement for stdin fgets()
 */
charptrT SANfgets(charptrT s, siT l)
	{
	return fgets(s, l, stdin);
	}

/*
 * Called when program initializes XBoard. 
 * Intended to provide switch to 'raw' mode, etc.
 */	
void SANXboardInit(void)
	{
	in_xboard = 1;
  signal(SIGINT, SIG_IGN);
	}

/*
 * Called when program exits XBoard
 * Intended to undo anything SANXboardInit() may do.
 */
void SANXboardTerm(void)
	{
	in_xboard = 0;
	/* undo signal() */
	}

/*
 * is_ready() -- return TRUE if file handle FD is ready
 *  (Assumes this is supported by compiler)
 */
#ifndef _WIN32
#include <sys/select.h>
#include <sys/time.h>

int is_ready(int fd)
{
    int rc;
    fd_set fds;
    struct timeval tv;

    FD_ZERO(&fds);
    FD_SET(fd,&fds);
    tv.tv_sec = tv.tv_usec = 0;

    rc = select(fd+1, &fds, NULL, NULL, &tv);
    if (rc < 0)
      return -1;

    return FD_ISSET(fd,&fds) ? 1 : 0;
}
#else
#include <conio.h>
int is_ready(int fd)
{
	int key=0;
	key = _kbhit();
    return key;
}
#endif	

/*<<< sanosi.c: EOF */
