/*>>> x_jester.c: Jester chess engine SAN kit interface */

/* Revised: 05-14-2001 11:42:44 */

/*
 * 02-12-2001 10:30:38 Modified for stdtype.h
 * 02-13-2001  9:28:13 Modified to set up board
 * 02-16-2001 10:00:00 Explicit cast on select() call
 * 03-09-2001 16:59:45 select() -> select_move()
 * 05-11-2001 10:44:14 use SAN Kit I/O fns
 * 05-14-2001 11:17:27 add JETAllowCalls() and JETRefuseCalls()
 */

#include <stdio.h>
#include <stdlib.h>

#include "stdtype.h"

#ifndef _LINUX
#include "SanKit\sandef.h"
#else
#include "SanKit/sandef.h"
#endif

#include "jetdef.h"

#ifndef _LINUX
#include "SanKit\sanvar.h"
#include "SanKit\sanatk.h"
#include "SanKit\sandsp.h"
#include "SanKit\sanmer.h"
#include "SanKit\sanmsc.h"
#include "SanKit\sansrt.h"
#else
#include "SanKit/sanvar.h"
#include "SanKit/sanatk.h"
#include "SanKit/sandsp.h"
#include "SanKit/sanmer.h"
#include "SanKit/sanmsc.h"
#include "SanKit/sansrt.h"
#endif



#include "x_jester.h"
#include "jester.h"

/*
 * Supplied so engine can use SAN Kit without callbacks.  
 */
static siT JETActive = 1;

/*---> JETAllowCalls: turn on SAN Kit callbacks */
nonstatic
void
JETAllowCalls(void)
	{
  if (optnv[optn_dbug])
    SANDspLine("JETAllowCalls()");
	JETActive = 1;
	}

/*---> JETRefuseCalls: turn off SAN Kit callbacks */
nonstatic
void
JETRefuseCalls(void)
	{
  if (optnv[optn_dbug])
    SANDspLine("JETRefuseCalls()");
	JETActive = 0;
	}

/*--> JETInit: one time call to initialize JET subsystem */
/* (Called once when program is started.) */
nonstatic
void
JETInit(void)
  {
  if (optnv[optn_dbug])
    SANDspLine("JETInit()");
	JETAllowCalls();
  initialize();
  return;
  }

/*--> JETTerm: one time call to terminate JET subsystem */
/* (Called once when `exit' command executed.) */
nonstatic
void
JETTerm(void)
  {
  if (optnv[optn_dbug])
    SANDspLine("JETTerm()");
  terminate();
  return;
  }

/*--> JETNewGame: send "new game" request to JET subsystem */
/* (Called when program starts or `newg' command executed.) */
nonstatic
void
JETNewGame(void)
  {
	if (!JETActive) 
		return;
  if (optnv[optn_dbug])
    SANDspLine("JETNewGame()");
  newg();
  return;
  }

/*--> JETSelect: select a move */
nonstatic
void
JETSelect(void)
  {
  mptrT mptr;
  siT done, from, i, to, promo;

	if (!JETActive) 
		return;
  if (optnv[optn_dbug])
    SANDspLine("JETSelect()");

  /* set default */
  done = 0;

  /* check if only one move */

  if (!done)
    if ((mptr = SANSingleMove()) != NULL)
      {
      select_m = *mptr;
      done = 1;
      }
  ;

  /* look for mating move */

  if (!done)
    if ((mptr = SANMatingMove()) != NULL)
      {
      select_m = *mptr;
      done = 1;
      }
  ;

  /* call engine to choose move otherwise */

  if (!done)
    {
    select_move((int *)&from, (int *)&to, (int *)&promo);
    curr_g.g_curr = curr_g.g_base;
    for (i = 0; i < curr_g.g_gmvc; i++)
      {
      if (curr_g.g_curr->m_frsq == from &&
        curr_g.g_curr->m_tosq == to)
        {
        if (!promo)
          {
          select_m = (*curr_g.g_curr);
          done = 1;
          break;
          }
        else if (curr_g.g_curr->m_scmv == (promo+3))
          {
          select_m = (*curr_g.g_curr);
          done = 1;
          break;
          }
        }
      curr_g.g_curr++;
      }
    }
  ;

  return;
  }

/*--> JETPlay: send "play move" request to JET subsystem */
nonstatic
void
JETPlay(void)
  {
	if (!JETActive) 
		return;
  if (optnv[optn_dbug])
    SANDspLine("JETPlay()");
  playmove(prev_m.m_frsq,
    prev_m.m_tosq,
    (prev_m.m_scmv < scmv_ppn ? 0 : prev_m.m_scmv-scmv_cqs));
  return;
  }

/*--> JETUnplay: send "unplay move" request to JET subsystem */
nonstatic
void
JETUnplay(void)
  {
	if (!JETActive) 
		return;
  if (optnv[optn_dbug])
    SANDspLine("JETUnplay()");
  unplaymove();
  return;
  }

/*--> JETSetPos: send "set position" request to JET subsystem */
nonstatic
void
JETSetPos(void)
  {
  sqT sq;
  cpT cp;
  siT castle;

	if (!JETActive) 
		return;
  if (optnv[optn_dbug])
    SANDspLine("JETSetPos()");

  /* easy scan board */
  for (sq = sq_a1; sq <= sq_h8; sq++)
    {
    cp = gb.bv[sq];
    if (cp == cp_v0)
      {
      placeboard(sq, EMPTY, EMPTY);
      }
    else
      {
      if (cp > cp_wk)
        placeboard(sq, DARK, cp-6);
      else
        placeboard(sq, LIGHT, cp);
      }
    }

  /* set side on move */
  placecolor((curr_e.e_actc == c_w) ? LIGHT : DARK);

  /* set castle permissions */
  castle = 0;
  castle += ((curr_e.e_cast & cast_wk) ? 1 : 0);
  castle += ((curr_e.e_cast & cast_wq) ? 2 : 0);
  castle += ((curr_e.e_cast & cast_bk) ? 4 : 0);
  castle += ((curr_e.e_cast & cast_bq) ? 8 : 0);
  placecastle(castle);

  /* set en passant target square */
  placeep(curr_e.e_epsq);

  /* set half and full move clocks */
  placeclocks(curr_e.e_hmvc, curr_e.e_fmvn);

  return;
  }

/*--> JETConfig: send "re-configuration" request to JET subsystem */
nonstatic
void
JETConfig(void)
  {
	if (!JETActive) 
		return;
  if (optnv[optn_dbug])
    SANDspLine("JETConfig()");
  sprintf(stv, "Playing level=%d plies\n", level);
	SANDspSTV();
  sprintf(stv, "Average response time=%d sec.\n", artsec);
	SANDspSTV();
  configure(level, artsec*1000);
  return;
  }
		
/*<<< x_jester.c: EOF */
